@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2013-2023 The Octave Project Developers
+// Copyright (C) 2013-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -37,9 +37,9 @@ class octave_txt_lexer : public QsciLexer
 
 public:
 
-  virtual const char * language (void) const;
+  virtual const char * language () const;
 
-  virtual const char * lexer (void) const;
+  virtual const char * lexer () const;
 
   virtual QString description (int style) const;
 };
