@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2011-2023 The Octave Project Developers
+// Copyright (C) 2011-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -32,26 +32,22 @@
 
 OCTAVE_BEGIN_NAMESPACE(octave)
 
-class base_qobject;
-
 class welcome_wizard : public QDialog
 {
   Q_OBJECT
 
 public:
 
-  typedef QWidget *(*page_creator_fptr) (base_qobject&, welcome_wizard *);
+  typedef QWidget *(*page_creator_fptr) (welcome_wizard *);
 
-  welcome_wizard (base_qobject& oct_qobj, QWidget *parent = nullptr);
+  welcome_wizard (QWidget *parent = nullptr);
 
-  ~welcome_wizard (void) = default;
+  ~welcome_wizard () = default;
 
-  void adjust_size (void);
+  void adjust_size ();
 
 private:
 
-  base_qobject& m_octave_qobj;
-
   QList<page_creator_fptr> m_page_ctor_list;
   QList<page_creator_fptr>::iterator m_page_list_iterator;
   QWidget *m_current_page;
@@ -63,11 +59,11 @@ public slots:
 
   void handle_web_connect_option (int state);
 
-  void show_page (void);
-  void previous_page (void);
-  void next_page (void);
+  void show_page ();
+  void previous_page ();
+  void next_page ();
 
-  void accept (void);
+  void accept ();
 };
 
 class initial_page : public QWidget
@@ -76,14 +72,14 @@ class initial_page : public QWidget
 
 public:
 
-  initial_page (base_qobject& oct_qobj, welcome_wizard *wizard);
+  initial_page (welcome_wizard *wizard);
 
-  ~initial_page (void) = default;
+  ~initial_page () = default;
 
   static QWidget *
-  create (base_qobject& oct_qobj, welcome_wizard *wizard)
+  create (welcome_wizard *wizard)
   {
-    return new initial_page (oct_qobj, wizard);
+    return new initial_page (wizard);
   }
 
 private:
@@ -101,14 +97,14 @@ class setup_community_news : public QWidget
 
 public:
 
-  setup_community_news (base_qobject& oct_qobj, welcome_wizard *wizard);
+  setup_community_news (welcome_wizard *wizard);
 
-  ~setup_community_news (void) = default;
+  ~setup_community_news () = default;
 
   static QWidget *
-  create (base_qobject& oct_qobj, welcome_wizard *wizard)
+  create (welcome_wizard *wizard)
   {
-    return new setup_community_news (oct_qobj, wizard);
+    return new setup_community_news (wizard);
   }
 
 private:
@@ -129,14 +125,14 @@ class final_page : public QWidget
 
 public:
 
-  final_page (base_qobject& oct_qobj, welcome_wizard *wizard);
+  final_page (welcome_wizard *wizard);
 
-  ~final_page (void) = default;
+  ~final_page () = default;
 
   static QWidget *
-  create (base_qobject& oct_qobj, welcome_wizard *wizard)
+  create (welcome_wizard *wizard)
   {
-    return new final_page (oct_qobj, wizard);
+    return new final_page (wizard);
   }
 
 private:
