@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2012-2023 The Octave Project Developers
+// Copyright (C) 2012-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -282,7 +282,7 @@ get_class_context (std::string& name, bool& in_constructor)
 }
 
 cdef_class
-get_class_context (void)
+get_class_context ()
 {
   std::string dummy_string;
   bool dummy_bool;
