@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1993-2023 The Octave Project Developers
+// Copyright (C) 1993-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -56,7 +56,7 @@
 
 OCTAVE_BEGIN_NAMESPACE(octave)
 
-cmdline_options::cmdline_options (void)
+cmdline_options::cmdline_options ()
 {
   m_all_args.resize (1);
   m_all_args[0] = "";
@@ -239,12 +239,13 @@ cmdline_options::cmdline_options (int argc, char **argv)
                                     argc-octave_optind_wrapper ());
 }
 
-octave_value cmdline_options::as_octave_value (void) const
+octave_value
+cmdline_options::as_octave_value () const
 {
   octave_scalar_map m;
 
   m.assign ("sys_argc", sys_argc ());
-  m.assign ("sys_argv", Cell (string_vector (sys_argv ())));
+  m.assign ("sys_argv", Cell (all_args ()));
   m.assign ("echo_commands", echo_commands ());
   m.assign ("forced_interactive", forced_interactive ());
   m.assign ("forced_line_editing", forced_line_editing ());
@@ -295,7 +296,7 @@ application::application (const cmdline_options& opts)
 // an explicit definition of the destructor here is much simpler than
 // including the full declaration of interpreter in the
 // octave.h header file.
-application::~application (void) { }
+application::~application () { }
 
 void
 application::set_program_names (const std::string& pname)
@@ -324,25 +325,29 @@ application::intern_argv (const string_vector& args)
     }
 }
 
-bool application::forced_interactive (void)
+bool
+application::forced_interactive ()
 {
   return s_instance ? s_instance->m_options.forced_interactive () : false;
 }
 
 // Provided for convenience.  Will be removed once we eliminate the
 // old terminal widget.
-bool application::experimental_terminal_widget (void) const
+bool
+application::experimental_terminal_widget () const
 {
   return (s_instance
           ? s_instance->m_options.experimental_terminal_widget () : false);
 }
 
-bool application::interpreter_initialized (void)
+bool
+application::interpreter_is_initialized () const
 {
-  return m_interpreter ? m_interpreter->initialized () : false;
+  return m_interpreter ? m_interpreter->is_initialized () : false;
 }
 
-interpreter& application::create_interpreter (void)
+interpreter&
+application::create_interpreter ()
 {
   if (! m_interpreter)
     m_interpreter = std::unique_ptr<interpreter> (new interpreter (this));
@@ -350,23 +355,27 @@ interpreter& application::create_interpreter (void)
   return *m_interpreter;
 }
 
-void application::initialize_interpreter (void)
+void
+application::initialize_interpreter ()
 {
   if (m_interpreter)
     m_interpreter->initialize ();
 }
 
-int application::execute_interpreter (void)
+int
+application::execute_interpreter ()
 {
   return m_interpreter ? m_interpreter->execute () : -1;
 }
 
-void application::delete_interpreter (void)
+void
+application::delete_interpreter ()
 {
   m_interpreter.reset ();
 }
 
-void application::init (void)
+void
+application::init ()
 {
   if (s_instance)
     throw std::runtime_error
@@ -420,7 +429,8 @@ void application::init (void)
   sysdep_init ();
 }
 
-int cli_application::execute (void)
+int
+cli_application::execute ()
 {
   interpreter& interp = create_interpreter ();
 
@@ -450,6 +460,27 @@ Return true if Octave is running in GUI mode and false otherwise.
 %!error <Invalid call> isguirunning (1)
 */
 
+DEFUN (__is_multi_threaded__, args, ,
+       doc: /* -*- texinfo -*-
+@deftypefn {} {@var{tf} =} __is_multi_threaded__ ()
+Return true if Octave is running with multiple threads.
+@seealso{isguirunning}
+@end deftypefn */)
+{
+  if (args.length () != 0)
+    print_usage ();
+
+  // FIXME: This isn't quite right, it just says that we intended to
+  // start the GUI, not that it is actually running.
+
+  return ovl (application::is_multi_threaded ());
+}
+
+/*
+%!assert (islogical (__is_multi_threaded__ ()))
+%!error <Invalid call> __is_multi_threaded__ (1)
+*/
+
 DEFUN (argv, args, ,
        doc: /* -*- texinfo -*-
 @deftypefn {} {@var{args} =} argv ()
