@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 2008-2019 Jaroslav Hajek
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2008-2024 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -31,7 +34,7 @@ along with Octave; see the file COPYING.  If not, see
 
 OCTAVE_NORETURN static
 void
-err_invalid_permutation (void)
+err_invalid_permutation ()
 {
   (*current_liboctave_error_handler) ("PermMatrix: invalid permutation vector");
 }
@@ -41,7 +44,7 @@ PermMatrix::setup (const Array<octave_idx_type>& p, bool colp, bool check)
 {
   if (check)
     {
-      if (! idx_vector (p).is_permutation (p.numel ()))
+      if (! octave::idx_vector (p).is_permutation (p.numel ()))
         err_invalid_permutation ();
     }
 
@@ -56,7 +59,7 @@ PermMatrix::PermMatrix (const Array<octave_idx_type>& p, bool colp, bool check)
 }
 
 void
-PermMatrix::setup (const idx_vector& idx, bool colp, octave_idx_type n)
+PermMatrix::setup (const octave::idx_vector& idx, bool colp, octave_idx_type n)
 {
   octave_idx_type len = idx.length (n);
 
@@ -71,7 +74,7 @@ PermMatrix::setup (const idx_vector& idx, bool colp, octave_idx_type n)
     *this = this->transpose ();
 }
 
-PermMatrix::PermMatrix (const idx_vector& idx, bool colp, octave_idx_type n)
+PermMatrix::PermMatrix (const octave::idx_vector& idx, bool colp, octave_idx_type n)
   : Array<octave_idx_type> ()
 {
   setup (idx, colp, n);
@@ -95,7 +98,7 @@ PermMatrix::checkelem (octave_idx_type i, octave_idx_type j) const
 }
 
 PermMatrix
-PermMatrix::transpose (void) const
+PermMatrix::transpose () const
 {
   octave_idx_type len = Array<octave_idx_type>::numel ();
 
@@ -108,13 +111,13 @@ PermMatrix::transpose (void) const
 }
 
 PermMatrix
-PermMatrix::inverse (void) const
+PermMatrix::inverse () const
 {
   return transpose ();
 }
 
 octave_idx_type
-PermMatrix::determinant (void) const
+PermMatrix::determinant () const
 {
   // Determine the sign of a permutation in linear time.
   // Is this widely known?
@@ -217,7 +220,7 @@ operator *(const PermMatrix& a, const PermMatrix& b)
   if (n != b.rows ())
     octave::err_nonconformant ("operator *", n, n, b.rows (), b.rows ());
 
-  r = PermMatrix (ia.index (idx_vector (ib)), true, false);
+  r = PermMatrix (ia.index (octave::idx_vector (ib)), true, false);
 
   return r;
 }
