@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2016-2020 The Octave Project Developers
+// Copyright (C) 2016-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -121,3 +121,19 @@ octave_optind_wrapper (void)
 {
   return optind;
 }
+
+int
+octave_get_opterr_wrapper (void)
+{
+  return opterr;
+}
+
+int
+octave_set_opterr_wrapper (int val)
+{
+  int retval = opterr;
+
+  opterr = val;
+
+  return retval;
+}
