@@ -123,12 +123,12 @@ endfunction
 %!assert (mad (123), 0)
 %!assert (mad (Inf), NaN)
 %!assert (mad ([3, Inf]),Inf)
-%!assert (mad ([0 0 1 2 100]), 31.76)
-%!assert (mad (single ([0 0 1 2 100])), single (31.76))
-%!assert (mad ([0 0 1 2 100]'), 31.76)
-%!assert (mad ([0 0 1 2 100], 1), 1)
-%!assert (mad (single ([0 0 1 2 100]), 1), single (1))
-%!assert (mad ([0 0 1 2 100]', 1), 1)
+%!assert (mad ([0, 0, 1, 2, 100]), 31.76)
+%!assert (mad (single ([0, 0, 1, 2, 100])), single (31.76))
+%!assert (mad ([0, 0, 1, 2, 100]'), 31.76)
+%!assert (mad ([0, 0, 1, 2, 100], 1), 1)
+%!assert (mad (single ([0, 0, 1, 2, 100]), 1), single (1))
+%!assert (mad ([0, 0, 1, 2, 100]', 1), 1)
 %!assert (mad (magic (4)), [4, 4, 4, 4])
 %!assert (mad (magic (4), [], 2), [6; 2; 2; 6])
 %!assert (mad (magic (4), 1), [2.5, 3.5, 3.5, 2.5])
@@ -141,22 +141,22 @@ endfunction
 %!assert (mad (magic (4), 0, "all"), 4)
 %!assert (mad (magic (4), 1, "all"), 4)
 %!assert (mad (magic (4), 0, [1 2]), 4)
-%!assert (mad (magic (4), 0, [1 3]), mad (magic(4), 0, 1))
+%!assert (mad (magic (4), 0, [1 3]), mad (magic (4), 0, 1))
 %!assert (mad (magic (4), 0, [1 2 3]), 4)
 %!assert (mad (magic (4), 1, [1 2]), 4)
-%!assert (mad (magic (4), 1, [1 3]), mad (magic(4), 1, 1))
+%!assert (mad (magic (4), 1, [1 3]), mad (magic (4), 1, 1))
 %!assert (mad (magic (4), 1, [1 2 3]), 4)
 %!assert (mad (magic (4), 0, [3 4 99]), zeros (4))
 %!assert (mad (magic (4), 1, [3 4 99]), zeros (4))
 
 ## Verify ignoring NaN values unless all NaN
 %!assert (mad (NaN), NaN)
-%!assert (mad (NaN (2)), NaN(1,2))
-%!assert (mad ([1,2;3,NaN]), [1, 0])
-%!assert (mad ([1,2;3,NaN], [], 1), [1, 0])
-%!assert (mad ([1,2;3,NaN], [], 2), [0.5; 0], eps)
-%!assert (mad ([1,NaN;3,NaN], [], 1), [1, NaN])
-%!assert (mad ([1,NaN;3,NaN], [], 2), [0; 0])
+%!assert (mad (NaN (2)), NaN (1, 2))
+%!assert (mad ([1, 2; 3, NaN]), [1, 0])
+%!assert (mad ([1, 2; 3, NaN], [], 1), [1, 0])
+%!assert (mad ([1, 2; 3, NaN], [], 2), [0.5; 0], eps)
+%!assert (mad ([1, NaN; 3, NaN], [], 1), [1, NaN])
+%!assert (mad ([1, NaN; 3, NaN], [], 2), [0; 0])
 
 ## Verify compatible empty handling
 %!assert (mad ([]), NaN)
@@ -164,24 +164,71 @@ endfunction
 %!assert (mad ([], 0, 2), NaN (0,1))
 %!assert (mad ([], 0, 3), NaN (0,0))
 %!assert (mad (single ([])), NaN ('single'))
-%!assert (mad (ones (0,1)), NaN)
-%!assert (mad (ones (0,1), 0, 1), NaN (1,1))
-%!assert (mad (ones (0,1), 0, 2), NaN (0,1))
-%!assert (mad (ones (0,1), 0, 3), NaN (0,1))
-%!assert (mad (ones (1,0)), NaN)
-%!assert (mad (ones (1,0), 0, 1), NaN (1,0))
-%!assert (mad (ones (1,0), 0, 2), NaN (1,1))
-%!assert (mad (ones (1,0), 0, 3), NaN (1,0))
-%!assert (mad (ones (0,0,0)), NaN (1,0,0))
-%!assert (mad (ones (1,0,0)), NaN (1,1,0))
-%!assert (mad (ones (0,1,0)), NaN (1,1,0))
-%!assert (mad (ones (0,0,0)), NaN (1,0,0))
-%!assert (mad (ones (0,1,0), 0, 1), NaN (1,1,0))
-%!assert (mad (ones (0,1,0), 0, 2), NaN (0,1,0))
-%!assert (mad (ones (0,1,0), 0, 3), NaN (0,1,1))
-%!assert (mad (ones (0,1,0), 0, 4), NaN (0,1,0))
-%!assert (mad (ones (0,2,1,0)), ones (1,2,1,0))
-%!assert (mad (ones (2,0,1,0)), ones (1,0,1,0))
+%!assert (mad (ones (0, 1)), NaN)
+%!assert (mad (ones (0, 1), 0, 1), NaN (1, 1))
+%!assert (mad (ones (0, 1), 0, 2), NaN (0, 1))
+%!assert (mad (ones (0, 1), 0, 3), NaN (0, 1))
+%!assert (mad (ones (1, 0)), NaN)
+%!assert (mad (ones (1, 0), 0, 1), NaN (1, 0))
+%!assert (mad (ones (1, 0), 0, 2), NaN (1, 1))
+%!assert (mad (ones (1, 0), 0, 3), NaN (1, 0))
+%!assert (mad (ones (0, 0, 0)), NaN (1, 0, 0))
+%!assert (mad (ones (1, 0, 0)), NaN (1, 1, 0))
+%!assert (mad (ones (0, 1, 0)), NaN (1, 1, 0))
+%!assert (mad (ones (0, 0, 0)), NaN (1, 0, 0))
+%!assert (mad (ones (0, 1, 0), 0, 1), NaN (1, 1, 0))
+%!assert (mad (ones (0, 1, 0), 0, 2), NaN (0, 1, 0))
+%!assert (mad (ones (0, 1, 0), 0, 3), NaN (0, 1, 1))
+%!assert (mad (ones (0, 1, 0), 0, 4), NaN (0, 1, 0))
+%!assert (mad (ones (0, 2, 1, 0)), ones (1, 2, 1, 0))
+%!assert (mad (ones (2, 0, 1, 0)), ones (1, 0, 1, 0))
+
+## Test Inf handling
+%!assert (mad ([3, 4, Inf]), Inf)
+%!assert (mad ([Inf, 3, 4]), Inf)
+%!assert (mad ([3, 4, Inf], 0), Inf)
+%!assert (mad ([3, 4, Inf], 0, 1), [0, 0, NaN])
+%!assert (mad ([3, 4, Inf], 0, 2), Inf)
+%!assert (mad ([3, 4, Inf], 0, 3), [0, 0, NaN])
+%!assert (mad ([3, 4, Inf]', 0), Inf)
+%!assert (mad ([3, 4, Inf]', 0, 1), Inf)
+%!assert (mad ([3, 4, Inf]', 0, 2), [0; 0; NaN])
+%!assert (mad ([3, 4, Inf]', 0, 3), [0; 0; NaN])
+
+%!assert (mad ([Inf, 3, 4], 1), 1)
+%!assert (mad ([3, 4, Inf], 1), 1)
+%!assert (mad ([3, 4, Inf], 1, 1), [0, 0, NaN])
+%!assert (mad ([3, 4, Inf], 1, 2), 1)
+%!assert (mad ([3, 4, Inf], 1, 3), [0, 0, NaN])
+%!assert (mad ([3, 4, Inf]', 1), 1)
+%!assert (mad ([3, 4, Inf]', 1, 1), 1)
+%!assert (mad ([3, 4, Inf]', 1, 2), [0; 0; NaN])
+%!assert (mad ([3, 4, Inf]', 1, 3), [0; 0; NaN])
+
+%!assert (mad ([3, Inf, Inf], 1), Inf)
+%!assert (mad ([3, 4, 5, Inf], 1), 1)
+%!assert (mad ([3, 4, Inf, Inf], 1), Inf)
+%!assert (mad ([3, Inf, Inf, Inf], 1), Inf)
+
+%!assert <*65405> (mad ([-Inf, Inf]), NaN)
+%!assert <*65405> (mad ([-Inf, Inf], 0), NaN)
+%!assert <*65405> (mad ([-Inf, Inf], 1), NaN)
+%!assert <*65405> (mad ([-Inf, Inf]', 0), NaN)
+%!assert <*65405> (mad ([-Inf, Inf]', 1), NaN)
+%!assert <*65405> (mad ([-Inf, Inf]', 0, 1), NaN)
+%!assert <*65405> (mad ([-Inf, Inf]', 0, 2), [NaN; NaN])
+%!assert <*65405> (mad ([-Inf, Inf]', 0, 3), [NaN; NaN])
+%!assert <*65405> (mad ([-Inf, Inf]', 1, 1), NaN)
+%!assert <*65405> (mad ([-Inf, Inf]', 1, 2), [NaN; NaN])
+%!assert <*65405> (mad ([-Inf, Inf]', 1, 3), [NaN; NaN])
+%!assert <*65405> (mad (Inf (2), 0), [NaN, NaN])
+%!assert <*65405> (mad (Inf (2), 1), [NaN, NaN])
+%!assert <*65405> (mad (Inf (2), 0, 1), [NaN, NaN])
+%!assert <*65405> (mad (Inf (2), 0, 2), [NaN; NaN])
+%!assert <*65405> (mad (Inf (2), 0, 3), NaN (2))
+%!assert <*65405> (mad (Inf (2), 1, 1), [NaN, NaN])
+%!assert <*65405> (mad (Inf (2), 1, 2), [NaN; NaN])
+%!assert <*65405> (mad (Inf (2), 1, 3), NaN (2))
 
 ## Test input case insensitivity
 %!assert (mad ([1 2 3], 0, "aLL"), 2/3, eps)
