@@ -1,4 +1,9 @@
-## Copyright (C) 2006-2018 Paul Kienzle
+########################################################################
+##
+## Copyright (C) 2006-2024 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{days} =} datenum (@var{datevec})
@@ -35,16 +42,22 @@
 ## The fractional part, @code{rem (@var{days}, 1)} corresponds to the time
 ## on the given day.
 ##
-## The input may be a date vector (see @code{datevec}),
-## datestr (see @code{datestr}), or directly specified as input.
+## The input may be a date vector (@pxref{XREFdatevec,,@code{datevec}}),
+## date string (@pxref{XREFdatestr,,@code{datestr}}), or directly specified
+## as input.
 ##
 ## When processing input datestrings, @var{f} is the format string used to
-## interpret date strings (see @code{datestr}).  If no format @var{f} is
-## specified, then a relatively slow search is performed through various
-## formats.  It is always preferable to specify the format string @var{f} if
-## it is known.  Formats which do not specify a particular time component
-## will have the value set to zero.  Formats which do not specify a date
-## will default to January 1st of the current year.
+## interpret date strings (@pxref{XREFdatestr,,@code{datestr}}).  If no
+## format @var{f} is specified, then a relatively slow search is performed
+## through various formats.  It is always preferable to specify the format
+## string @var{f} if it is known.  Formats which do not specify a particular
+## time component will have the value set to zero.  Formats which do not
+## specify a date will default to January 1st of the current year.
+##
+## When passing separate @var{year}, @var{month}, @var{day}, etc.@: arguments,
+## each may be a scalar or nonscalar array.  Nonscalar inputs must all be of
+## the same size.  Scalar inputs will be expanded to be the size of the
+## nonscalar inputs.
 ##
 ## @var{p} is the year at the start of the century to which two-digit years
 ## will be referenced.  If not specified, it defaults to the current year
@@ -75,6 +88,57 @@
 ## Days can be fractional.
 ## @end itemize
 ##
+## Examples:
+##
+## @example
+## @group
+## Convert from datestrs:
+## d = datenum ("1966-06-14")
+## @result{} d = 718232
+## @end group
+##
+## @group
+## d = datenum (@{"1966-06-14", "1966-06-15", "1966-06-16"@})
+## @result{} d =
+##       718232
+##       718233
+##       718234
+## @end group
+##
+## @group
+## Convert from datevec:
+## d = datenum ([1966 06 14])
+## @result{} d = 718232
+## @end group
+##
+## @group
+## d = datenum ([1966 06 14 23 59 59])
+## @result{} d = 718232.9999884259
+## @end group
+##
+## @group
+## Specify date components separately:
+## d = datenum (1966, 6, 14)
+## @result{} d = 718232
+## @end group
+##
+## @group
+## d = datenum (1966, magic(3), 1)
+## @result{} d =
+##
+##       718280   718068   718219
+##       718127   718188   718249
+##       718158   718311   718099
+## @end group
+## @end example
+##
+## @strong{Caution:} datenums represent a specific time for the Earth as a
+## whole.  They do not take in to account time zones (shifts in time based
+## on location), nor seasonal changes due to Daylight Savings Time (shifts in
+## time based on local regulation).  Be aware that it is possible to create
+## datenums that, when interpreted by a function which accounts for time zone
+## and DST shifts such as @code{datestr}, are nonexistent or ambiguous.
+##
 ## @strong{Caution:} this function does not attempt to handle Julian calendars
 ## so dates before October 15, 1582 are wrong by as much as eleven days.  Also,
 ## be aware that only Roman Catholic countries adopted the calendar in 1582.
@@ -83,11 +147,11 @@
 ##
 ## @strong{Warning:} leap seconds are ignored.  A table of leap seconds is
 ## available on the Wikipedia entry for leap seconds.
+##
 ## @seealso{datestr, datevec, now, clock, date}
 ## @end deftypefn
-
+##
 ## Algorithm: Peter Baum (http://vsg.cape.com/~pbaum/date/date0.htm)
-## Author: pkienzle <pkienzle@users.sf.net>
 
 function [days, secs] = datenum (year, month = [], day = [], hour = 0, minute = 0, second = 0)
 
@@ -95,8 +159,7 @@ function [days, secs] = datenum (year, month = [], day = [], hour = 0, minute =
   persistent monthstart = [306; 337; 0; 31; 61; 92; 122; 153; 184; 214; 245; 275];
   persistent monthlength = [31; 28; 31; 30; 31; 30; 31; 31; 30; 31; 30; 31];
 
-  if (nargin == 0 || nargin > 6
-      || (nargin > 2 && (ischar (year) || iscellstr (year))))
+  if (nargin == 0 || (nargin > 2 && (ischar (year) || iscellstr (year))))
     print_usage ();
   endif
 
@@ -122,8 +185,7 @@ function [days, secs] = datenum (year, month = [], day = [], hour = 0, minute =
       endif
       ## Preserve shape if necessary, and work with column vectors
       ## for the remainder of the function.
-      do_reshape = (columns (year) > 1 || columns (month) > 1
-                    || columns (day) > 1);
+      do_reshape = ! iscolumn (day);
       if (do_reshape)
         sz_reshape = size (day);
         year = year(:);
@@ -178,7 +240,7 @@ function [days, secs] = datenum (year, month = [], day = [], hour = 0, minute =
   days = day + (hour + (minute + second/60)/60)/24;
 
   ## Output seconds if asked so that etime can be more accurate
-  if (isargout (2))
+  if (nargout > 1)
     secs = day*86400 + hour*3600 + minute*60 + second;
   endif
 
@@ -203,6 +265,8 @@ endfunction
 %! n = n';
 %! assert (datenum (t(1,:), t(2,:), t(3,:), t(4,:), t(5,:), t(6,:)), n, 2*eps);
 
+%!assert (size (datenum (2000, 1, ones(1, 1, 3))), [1 1 3])
+
 ## Test fractional years including leap years
 %!assert (fix (datenum ([2001.999 1 1; 2001.999 2 1])), [731216; 731247])
 %!assert (fix (datenum ([2004.999 1 1; 2004.999 2 1])), [732312; 732343])
@@ -216,15 +280,19 @@ endfunction
 %!assert (datenum ([2008;2009],1,1), [datenum(2008,1,1);datenum(2009,1,1)])
 %!assert (datenum (2008, [1;2], 1), [datenum(2008,1,1);datenum(2008,2,1)])
 %!assert (datenum (2008, 1, [1;2]), [datenum(2008,1,1);datenum(2008,1,2)])
-%!assert (datenum ([2008;2009], [1;2], 1), [datenum(2008,1,1);datenum(2009,2,1)])
-%!assert (datenum ([2008;2009], 1, [1;2]), [datenum(2008,1,1);datenum(2009,1,2)])
+%!assert (datenum ([2008;2009], [1;2], 1),
+%!        [datenum(2008,1,1);datenum(2009,2,1)])
+%!assert (datenum ([2008;2009], 1, [1;2]),
+%!        [datenum(2008,1,1);datenum(2009,1,2)])
 %!assert (datenum (2008, [1;2], [1;2]), [datenum(2008,1,1);datenum(2008,2,2)])
 ## And the other orientation
 %!assert (datenum ([2008 2009], 1, 1), [datenum(2008,1,1) datenum(2009,1,1)])
 %!assert (datenum (2008, [1 2], 1), [datenum(2008,1,1) datenum(2008,2,1)])
 %!assert (datenum (2008, 1, [1 2]), [datenum(2008,1,1) datenum(2008,1,2)])
-%!assert (datenum ([2008 2009], [1 2], 1), [datenum(2008,1,1) datenum(2009,2,1)])
-%!assert (datenum ([2008 2009], 1, [1 2]), [datenum(2008,1,1) datenum(2009,1,2)])
+%!assert (datenum ([2008 2009], [1 2], 1),
+%!        [datenum(2008,1,1) datenum(2009,2,1)])
+%!assert (datenum ([2008 2009], 1, [1 2]),
+%!        [datenum(2008,1,1) datenum(2009,1,2)])
 %!assert (datenum (2008, [1 2], [1 2]), [datenum(2008,1,1) datenum(2008,2,2)])
 
 ## Test string and cellstr inputs
@@ -237,8 +305,7 @@ endfunction
 %!assert (datenum ("5-19, 2001", "mm-dd, yyyy"), 730990)
 
 ## Test input validation
-%!error datenum ()
-%!error datenum (1,2,3,4,5,6,7)
+%!error <Invalid call> datenum ()
 %!error <expected date vector containing> datenum ([1, 2])
 %!error <expected date vector containing> datenum ([1,2,3,4,5,6,7])
 %!error <all inputs must be of class double> datenum (int32 (2000), int32 (1), int32 (1))
